@@ -14,11 +14,7 @@ use crate::{
     task_expanders,
     resolver::Adaptor,
     resolver::AsyncHyperResolver,
-    resolver::Resolver
-};
-
-use std::{
-    net::{IpAddr},
+    resolver::Resolver,
 };
 
 #[derive(Clone)]
@@ -117,7 +113,11 @@ impl<JS: JobStateValues, TS: TaskStateValues> JobRules<JS, TS> for CrawlingRules
         let options = &self.options;
         let mut task_filters: TaskFilters<JS, TS> = vec![
             Box::new(task_filters::MaxRedirect::new(options.max_redirect)),
-            Box::new(task_filters::SameDomain::new(options.allow_www)),
+            Box::new(task_filters::SelectiveTaskFilter::new(
+                // this filter is enabled only for links that we follow(either directly with get or with head+get)
+                vec![LinkTarget::Follow, LinkTarget::HeadFollow],
+                task_filters::SameDomain::new(options.allow_www))
+            ),
             Box::new(task_filters::HashSetDedup::new()),
         ];
         if options.page_budget.is_some() {
@@ -179,10 +179,7 @@ struct HttpClientFactory<R: Resolver> {
     networking_profile: config::ResolvedNetworkingProfile<R>,
 }
 
-type HttpClient<R> = hyper::Client<HttpConnector<R>>;
-type HttpConnector<R> = hyper_tls::HttpsConnector<hyper_utils::CountingConnector<hyper::client::HttpConnector<Adaptor<R>>>>;
-
-impl<R: Resolver> HttpClientFactory<R> {
+impl<R: Resolver> ClientFactory<R> for HttpClientFactory<R> {
     fn make(&self) -> (HttpClient<R>, hyper_utils::Stats) {
         let v = self.networking_profile.values.clone();
 
@@ -272,7 +269,7 @@ impl<R: Resolver> Crawler<R> {
         update_tx: Sender<JobUpdate<JS, TS>>,
     ) -> PinnedTask
     {
-        TracingTask::new(span!(Level::ERROR, url=%job.url), async move {
+        TracingTask::new(span!(url=%job.url), async move {
             let job = ResolvedJob::from(job);
 
             let scheduler = TaskScheduler::new(
@@ -286,13 +283,13 @@ impl<R: Resolver> Crawler<R> {
 
             let mut processor_handles = vec![];
             for i in 0..job.settings.concurrency {
-                let cf = client_factory.clone();
                 let mut processor = TaskProcessor::new(
                     job.clone(),
                     scheduler.job_update_tx.clone(),
                     scheduler.tasks_rx.clone(),
                     self.parse_tx.clone(),
-                    Box::new(move || cf.make())
+                    Box::new(client_factory.clone()),
+                    Arc::clone(&self.networking_profile.resolver),
                 );
 
                 processor_handles.push(tokio::spawn(async move {
@@ -347,7 +344,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> MultiCrawler<JS, TS,
     }
 
     pub fn go(self) -> PinnedTask<'static> {
-        TracingTask::new(span!(Level::ERROR), async move {
+        TracingTask::new(span!(), async move {
             let mut handles = vec![];
             for _ in 0..self.concurrency_profile.domain_concurrency {
                 let h = tokio::spawn({
