@@ -51,7 +51,7 @@ impl<JS: JobStateValues, TS: TaskStateValues> TaskScheduler<JS, TS> {
     fn schedule_filter(&mut self, task: &mut Task) -> task_filters::Action {
         let action = if task.link.redirect > 0 { "[scheduling redirect]" } else { "[scheduling]" };
 
-        let _span = span!(Level::ERROR, task = %task).entered();
+        let _span = span!(task = %task).entered();
         for filter in &mut self.task_filters {
             match filter.accept(&mut self.job.ctx, self.task_seq_num, task) {
                 task_filters::Action::Accept => continue,
@@ -82,7 +82,7 @@ impl<JS: JobStateValues, TS: TaskStateValues> TaskScheduler<JS, TS> {
         self.pages_pending -= 1;
         self.task_seq_num += 1;
 
-        if let (JobStatus::Processing(ref r), false) = (&task_response.status, ignore_links) {
+        if let (JobStatus::Processing(Ok(ref r)), false) = (&task_response.status, ignore_links) {
             let tasks :Vec<_> = r.links.iter()
                 .filter_map(|link| {
                     Task::new(Arc::clone(link), &task_response.task).ok()
@@ -112,7 +112,7 @@ impl<JS: JobStateValues, TS: TaskStateValues> TaskScheduler<JS, TS> {
     pub(crate) fn go<'a>(mut self) -> Result<PinnedTask<'a, JobUpdate<JS, TS>>> {
         let mut root_task = Task::new_root(&self.job.url)?;
 
-        Ok(TracingTask::new(span!(Level::ERROR, url = %self.job.url), async move {
+        Ok(TracingTask::new(span!(url = %self.job.url), async move {
             trace!(
                 soft_timeout_ms = self.job.settings.job_soft_timeout.as_millis() as u32,
                 hard_timeout_ms = self.job.settings.job_hard_timeout.as_millis() as u32,
@@ -152,7 +152,7 @@ impl<JS: JobStateValues, TS: TaskStateValues> TaskScheduler<JS, TS> {
             } else if is_soft_timeout {
                 Err(JobError::JobFinishedBySoftTimeout)
             } else {
-                Ok(JobData{})
+                Ok(JobFinished {})
             };
             Ok(JobUpdate {
                 task: root_task,
